@@ -18,11 +18,11 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemBasicPlugin.h 3717 2006-11-15 17:21:16Z chpe $
+ * $Id: totemBasicPlugin.h 4625 2007-08-29 15:38:18Z hadess $
  */
 
-#ifndef __GMP_PLAYER_H__
-#define __GMP_PLAYER_H__
+#ifndef __BASIC_PLUGIN_H__
+#define __BASIC_PLUGIN_H__
 
 #include <nsIClassInfo.h>
 
@@ -50,4 +50,4 @@ class totemScriptablePlugin : public totemIBasicPlayer,
     totemPlugin *mPlugin;
 };
 
-#endif /* __GMP_PLAYER_H__ */
+#endif /* __BASIC_PLUGIN_H__ */
