@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemGMPPlugin.cpp 4361 2007-06-06 13:07:55Z hadess $
+ * $Id: totemGMPPlugin.cpp 4846 2007-11-01 17:29:11Z hadess $
  */
 
 #include <mozilla-config.h>
@@ -67,7 +67,8 @@ static const totemPluginMimeEntry kMimeTypes[] = {
 	{ "video/x-wmv", "wmv", "video/x-ms-wmv" },
 	{ "video/x-ms-wvx", "wmv", "video/x-ms-wmv" },
 	{ "video/x-ms-wm", "wmv", "video/x-ms-wmv" },
-	{ "application/x-ms-wms", "wms", "video/x-ms-wmv" }
+	{ "application/x-ms-wms", "wms", "video/x-ms-wmv" },
+	{ "application/asx", "asx", "audio/x-ms-asx" }
 };
 
 void*
@@ -393,19 +394,25 @@ totemScriptablePlugin::SetUiMode(const nsACString & aUiMode)
 
 /* attribute AUTF8String URL; */
 NS_IMETHODIMP 
-totemScriptablePlugin::GetURL(nsACString & aURL)
+totemScriptablePlugin::GetURL(nsACString & _retval)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED ();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  return NS_ERROR_NOT_IMPLEMENTED;
+  _retval.Assign (mSrc);
+  return NS_OK;
 }
 
 NS_IMETHODIMP 
 totemScriptablePlugin::SetURL(const nsACString & aURL)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED ();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_STATE (IsValid ());
+
+  nsresult rv = mPlugin->SetSrc (aURL);
+
+  mSrc = aURL;
+  return rv;
 }
 
 /* readonly attribute AUTF8String versionInfo; */
@@ -567,9 +574,11 @@ totemScriptablePlugin::SetCurrentMarker(PRInt32 aCurrentMarker)
 NS_IMETHODIMP 
 totemScriptablePlugin::GetCurrentPosition(double *aCurrentPosition)
 {
-  TOTEM_SCRIPTABLE_WARN_UNIMPLEMENTED ();
+  TOTEM_SCRIPTABLE_LOG_ACCESS ();
 
-  *aCurrentPosition = 0.0;
+  NS_ENSURE_STATE (IsValid ());
+
+  *aCurrentPosition = mPlugin->mTime / 1000;
   return NS_OK;
 }
 
