@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemMullYPlugin.cpp,v 1.1 2006/09/07 22:11:44 hadess Exp $
+ * $Id: totemMullYPlugin.cpp 4180 2007-04-01 23:37:46Z hadess $
  */
 
 #include <mozilla-config.h>
@@ -32,8 +32,14 @@
 #include <nsMemory.h>
 #include <nsXPCOM.h>
 
+#define GNOME_ENABLE_DEBUG 1
+/* define GNOME_ENABLE_DEBUG for more debug spew */
+#include "debug.h"
+
 #include "totemIMullYPlayer.h"
 
+#include "totemClassInfo.h"
+
 #include "totemMullYPlugin.h"
 
 /* 67DABFBF-D0AB-41fa-9C46-CC0F21721616 */
@@ -51,12 +57,12 @@ static const totemPluginMimeEntry kMimeTypes[] = {
 totemScriptablePlugin::totemScriptablePlugin (totemPlugin *aPlugin)
   : mPlugin(aPlugin)
 {
-  g_print ("%s ctor [%p]\n", kClassDescription, (void*) this);
+  D ("%s ctor [%p]", kClassDescription, (void*) this);
 }
 
 totemScriptablePlugin::~totemScriptablePlugin ()
 {
-  g_print ("%s dtor [%p]\n", kClassDescription, (void*) this);
+  D ("%s dtor [%p]", kClassDescription, (void*) this);
 }
 
 /* static */ char *
@@ -81,120 +87,35 @@ NS_IMPL_ISUPPORTS2 (totemScriptablePlugin,
 
 /* nsIClassInfo */
 
-NS_IMETHODIMP
-totemScriptablePlugin::GetFlags (PRUint32 *aFlags)
-{
-  *aFlags = nsIClassInfo::PLUGIN_OBJECT | nsIClassInfo::DOM_OBJECT;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetImplementationLanguage (PRUint32 *aImplementationLanguage)
-{
-  *aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetInterfaces (PRUint32 *count,
-				      nsIID * **array)
-{
-  *array = NS_STATIC_CAST (nsIID**, nsMemory::Alloc (sizeof (nsIID)));
-  if (!*array)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  *count = 1;
-
-  (*array)[0] = NS_STATIC_CAST (nsIID*,
-  				nsMemory::Clone (&NS_GET_IID (totemIMullYPlayer),
-						 sizeof(nsIID)));
-  if (!(*array)[0]) {
-    NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY (0, *array);
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  g_message ("GetInterfaces");
-  return NS_OK;
-}
-     
-NS_IMETHODIMP
-totemScriptablePlugin::GetHelperForLanguage (PRUint32 language,
-					     nsISupports **_retval)
-{
-  *_retval = nsnull;
-  g_message ("GetHelperForLanguage %d", language);
-  return NS_OK;
-}
-     
-NS_IMETHODIMP
-totemScriptablePlugin::GetContractID (char * *aContractID)
-{
-  *aContractID = nsnull;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetClassDescription (char * *aClassDescription)
-{
-  *aClassDescription = NS_STATIC_CAST (char*,
-				       nsMemory::Clone (kClassDescription,
-						        sizeof (kClassDescription)));
-  if (!*aClassDescription)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  g_message ("GetClassDescription: %s", *aClassDescription);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetClassID (nsCID * *aClassID)
-{
-  *aClassID = NS_STATIC_CAST (nsCID*,
-			      nsMemory::Clone (&kClassID,
-					       sizeof (nsCID*)));
-  if (!*aClassID)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  g_message ("GetClassID");
-  return NS_OK;
-}
-     
-NS_IMETHODIMP
-totemScriptablePlugin::GetClassIDNoAlloc (nsCID *aClassIDNoAlloc)
-{
-  *aClassIDNoAlloc = kClassID;
-  g_message ("GetClassIDNoAlloc");
-  return NS_OK;
-}
+TOTEM_CLASSINFO_BEGIN (totemScriptablePlugin,
+		       1,
+		       kClassID,
+		       kClassDescription)
+  TOTEM_CLASSINFO_ENTRY (0, totemIMullYPlayer)
+TOTEM_CLASSINFO_END
 
 /* totemIMullYPlayer */
 
 NS_IMETHODIMP
 totemScriptablePlugin::Rewind ()
 {
-  NS_ENSURE_STATE (mPlugin);
+  NS_ENSURE_STATE (IsValid ());
 
-  totem_plugin_pause (mPlugin);
-
-  return NS_OK;
+  return mPlugin->DoCommand (TOTEM_COMMAND_PAUSE);
 }
 
 NS_IMETHODIMP
 totemScriptablePlugin::Play ()
 {
-  NS_ENSURE_STATE (mPlugin);
+  NS_ENSURE_STATE (IsValid ());
 
-  totem_plugin_play (mPlugin);
-
-  return NS_OK;
+  return mPlugin->DoCommand (TOTEM_COMMAND_PLAY);
 }
 
 NS_IMETHODIMP
 totemScriptablePlugin::Stop ()
 {
-  NS_ENSURE_STATE (mPlugin);
+  NS_ENSURE_STATE (IsValid ());
 
-  totem_plugin_stop (mPlugin);
-
-  return NS_OK;
+  return mPlugin->DoCommand (TOTEM_COMMAND_STOP);
 }
