@@ -36,6 +36,7 @@
 #include "totem-private.h"
 #include "totem-plugins-engine.h"
 #include "ev-sidebar.h"
+#include "totem-playlist.h"
 
 enum {
 	PROP_0,
@@ -233,6 +234,37 @@ totem_get_current_time (Totem *totem)
 	return bacon_video_widget_get_current_time (totem->bvw);
 }
 
+void
+totem_add_to_playlist_and_play (Totem *totem,
+				const char *uri,
+				const char *display_name,
+				gboolean add_to_recent)
+{
+	gboolean playlist_changed;
+	guint end;
+
+	totem_signal_block_by_data (totem->playlist, totem);
+
+	end = totem_playlist_get_last (totem->playlist);
+	playlist_changed = totem_playlist_add_mrl_with_cursor (totem->playlist, uri, display_name);
+	if (add_to_recent != FALSE)
+		gtk_recent_manager_add_item (totem->recent_manager, uri);
+
+	totem_signal_unblock_by_data (totem->playlist, totem);
+
+	if (playlist_changed)
+	{
+		char *mrl, *subtitle;
+
+		subtitle = NULL;
+		totem_playlist_set_current (totem->playlist, end + 1);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		totem_action_set_mrl_and_play (totem, mrl, subtitle);
+		g_free (mrl);
+		g_free (subtitle);
+	}
+}
+
 guint
 totem_get_playlist_length (Totem *totem)
 {
@@ -359,7 +391,7 @@ totem_remote_command_get_type (void)
 			ENUM_ENTRY (TOTEM_REMOTE_COMMAND_EJECT, "Eject"),
 			ENUM_ENTRY (TOTEM_REMOTE_COMMAND_PLAY_DVD, "Play DVD"),
 			ENUM_ENTRY (TOTEM_REMOTE_COMMAND_MUTE, "Mute"),
-			{ 0, 0, 0 }
+			{ 0, NULL, NULL }
 		};
 
 		etype = g_enum_register_static ("TotemRemoteCommand", values);
@@ -390,7 +422,7 @@ totem_disc_media_type_get_type (void)
 			ENUM_ENTRY (MEDIA_TYPE_CDDA, "CDDA disc"),
 			ENUM_ENTRY (MEDIA_TYPE_VCD, "VCD"),
 			ENUM_ENTRY (MEDIA_TYPE_DVD, "DVD"),
-			{ 0, 0, 0 }
+			{ 0, NULL, NULL }
 		};
 
 		etype = g_enum_register_static ("TotemDiscMediaType", values);
