@@ -27,28 +27,38 @@
 
 #include "config.h"
 
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #ifndef HAVE_GTK_ONLY
 #include <libgnomeui/gnome-authentication-manager.h>
 #endif
 #include <libgnomevfs/gnome-vfs-init.h>
 #include "bacon-video-widget.h"
+#include "totem-resources.h"
 
 /* #define THUMB_DEBUG */
 
-#define MIN_LEN_FOR_SEEK 25000
-#define HALF_SECOND G_USEC_PER_SEC * .5
+#ifdef G_HAVE_ISO_VARARGS
+#define PROGRESS_DEBUG(...) { if (verbose != FALSE) g_message (__VA_ARGS__); }
+#elif defined(G_HAVE_GNUC_VARARGS)
+#define PROGRESS_DEBUG(format...) { if (verbose != FALSE) g_message (format); }
+#endif
+
 #define BORING_IMAGE_VARIANCE 256.0		/* Tweak this if necessary */
 
-gboolean finished = FALSE;
 static gboolean jpeg_output = FALSE;
 static gboolean output_size = 128;
 static gboolean time_limit = TRUE;
+static gboolean verbose = FALSE;
+static gboolean g_fatal_warnings = FALSE;
+static gint64 second_index = -1;
 static char **filenames = NULL;
 
 #ifdef THUMB_DEBUG
@@ -79,8 +89,7 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 	holes = gdk_pixbuf_new_from_file (filename, NULL);
 	g_free (filename);
 
-	if (holes == NULL)
-	{
+	if (holes == NULL) {
 		g_object_ref (pixbuf);
 		return pixbuf;
 	}
@@ -92,9 +101,9 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 	for (i = 0; i < height; i += gdk_pixbuf_get_height (holes))
 	{
 		gdk_pixbuf_composite (holes, target, 0, i,
-				MIN (width, gdk_pixbuf_get_width (holes)),
-				MIN (height-i, gdk_pixbuf_get_height (holes)),
-				0, i, 1, 1, GDK_INTERP_NEAREST, 255);
+				      MIN (width, gdk_pixbuf_get_width (holes)),
+				      MIN (height - i, gdk_pixbuf_get_height (holes)),
+				      0, i, 1, 1, GDK_INTERP_NEAREST, 255);
 	}
 
 	tmp = gdk_pixbuf_flip (holes, FALSE);
@@ -104,11 +113,11 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 	for (i = 0; i < height; i += gdk_pixbuf_get_height (holes))
 	{
 		gdk_pixbuf_composite (holes, target,
-				width - gdk_pixbuf_get_width (holes), i,
-				MIN (width, gdk_pixbuf_get_width (holes)),
-				MIN (height-i, gdk_pixbuf_get_height (holes)),
-				width - gdk_pixbuf_get_width (holes), i,
-				1, 1, GDK_INTERP_NEAREST, 255);
+				      width - gdk_pixbuf_get_width (holes), i,
+				      MIN (width, gdk_pixbuf_get_width (holes)),
+				      MIN (height - i, gdk_pixbuf_get_height (holes)),
+				      width - gdk_pixbuf_get_width (holes), i,
+				      1, 1, GDK_INTERP_NEAREST, 255);
 	}
 
 	g_object_unref (holes);
@@ -231,16 +240,16 @@ is_image_interesting (GdkPixbuf *pixbuf)
 
 	/* Iterate through the image to calculate x-bar */
 	for (i = 0; i < num_samples; i++) {
-		x_bar += (float)buffer[i];
+		x_bar += (float) buffer[i];
 	}
-	x_bar /= ((float)num_samples);
+	x_bar /= ((float) num_samples);
 
 	/* Calculate the variance */
 	for (i = 0; i < num_samples; i++) {
-		float tmp = ((float)buffer[i] - x_bar);
+		float tmp = ((float) buffer[i] - x_bar);
 		variance += tmp * tmp;
 	}
-	variance /= ((float)(num_samples - 1));
+	variance /= ((float) (num_samples - 1));
 
 	return (variance > BORING_IMAGE_VARIANCE);
 }
@@ -320,25 +329,102 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	g_object_unref (with_holes);
 }
 
-static gpointer
-time_monitor (gpointer data)
+static GdkPixbuf *
+capture_interesting_frame (BaconVideoWidget * bvw, char *input, char *output) 
 {
-	g_usleep (30 * G_USEC_PER_SEC);
+	GdkPixbuf* pixbuf;
+	guint current;
+	GError *err = NULL;
+	const double frame_locations[] = {
+		1.0 / 3.0,
+		2.0 / 3.0,
+		0.1,
+		0.9,
+		0.5
+	};
 
-	if (finished != FALSE)
-		g_thread_exit (NULL);
+	/* Test at multiple points in the file to see if we can get an 
+	 * interesting frame */
+	for (current = 0; current < G_N_ELEMENTS(frame_locations); current++)
+	{
+		PROGRESS_DEBUG("About to seek to %f\n", frame_locations[current]);
+		if (bacon_video_widget_seek (bvw, frame_locations[current], NULL) == FALSE) {
+			bacon_video_widget_play (bvw, NULL);
+		}
 
-	g_print ("totem-video-thumbnailer couln't thumbnail file: '%s'\n"
-			"Reason: Took too much time to thumbnail.\n",
-			(const char *) data);
-	exit (0);
+		if (bacon_video_widget_can_get_frames (bvw, &err) == FALSE)
+		{
+			g_print ("totem-video-thumbnailer: '%s' isn't thumbnailable\n"
+				 "Reason: %s\n",
+				 input, err ? err->message : "programming error");
+			bacon_video_widget_close (bvw);
+			gtk_widget_destroy (GTK_WIDGET (bvw));
+			g_error_free (err);
+
+			exit (1);
+		}
+
+		/* Pull the frame, if it's interesting we bail early */
+		PROGRESS_DEBUG("About to get frame for iter %d\n", current);
+		pixbuf = bacon_video_widget_get_current_frame (bvw);
+		if (pixbuf != NULL && is_image_interesting (pixbuf) != FALSE) {
+			PROGRESS_DEBUG("Frame for iter %d is interesting\n", current);
+			break;
+		}
+
+		/* If we get to the end of this loop, we'll end up using
+		 * the last image we pulled */
+		if (current + 1 < G_N_ELEMENTS(frame_locations)) {
+			if (pixbuf != NULL) {
+				g_object_unref (pixbuf);
+				pixbuf = NULL;
+			}
+		}
+		PROGRESS_DEBUG("Frame for iter %d was not interesting\n", current);
+	}
+	return pixbuf;
+}
+
+static GdkPixbuf *
+capture_frame_at_time(BaconVideoWidget *bvw, char *input, char *output,  gint64 seconds) 
+{
+	GError *err = NULL;
+
+	if (bacon_video_widget_seek_time (bvw, seconds * 1000, &err) == FALSE) {
+		g_print ("totem-video-thumbnailer: could not seek to %d seconds in '%s'\n"
+			 "Reason: %s\n",
+			 (int) seconds, input, err ? err->message : "programming error");
+		bacon_video_widget_close (bvw);
+		gtk_widget_destroy (GTK_WIDGET (bvw));
+		g_error_free (err);
+
+		exit (1);
+	}
+	if (bacon_video_widget_can_get_frames (bvw, &err) == FALSE)
+	{
+		g_print ("totem-video-thumbnailer: '%s' isn't thumbnailable\n"
+			 "Reason: %s\n",
+			 input, err ? err->message : "programming error");
+		bacon_video_widget_close (bvw);
+		gtk_widget_destroy (GTK_WIDGET (bvw));
+		g_error_free (err);
+
+		exit (1);
+	}
+
+	return bacon_video_widget_get_current_frame (bvw);
 }
 
 static const GOptionEntry entries[] = {
 	{ "jpeg", 'j',  0, G_OPTION_ARG_NONE, &jpeg_output, "Output the thumbnail as a JPEG instead of PNG", NULL },
 	{ "size", 's', 0, G_OPTION_ARG_INT, &output_size, "Size of the thumbnail in pixels", NULL },
 	{ "no-limit", 'l', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &time_limit, "Don't limit the thumbnailing time to 30 seconds", NULL },
-	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, "Movies to index", NULL },
+	{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, "Output debug information", NULL },
+	{ "time", 't', 0, G_OPTION_ARG_INT64, &second_index, "Choose this time (in seconds) as the thumbnail", NULL },
+#ifndef THUMB_DEBUG
+	{"g-fatal-warnings", 0, 0, G_OPTION_ARG_NONE, &g_fatal_warnings, "Make all warnings fatal", NULL},
+#endif /* THUMB_DEBUG */
+ 	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, NULL, "[FILE...]" },
 	{ NULL }
 };
 
@@ -349,40 +435,46 @@ int main (int argc, char *argv[])
 	GError *err = NULL;
 	BaconVideoWidget *bvw;
 	GdkPixbuf *pixbuf;
-	int length;
 	char *input, *output;
 
-	const float frame_locations[] = { 1.0 / 3.0, 2.0 / 3.0, 0.1,0.9, 0.5 };
-	const int frame_locations_length = 5;
-	int current;
-
 #ifdef G_OS_UNIX
 	nice (20);
 #endif
 
+	g_thread_init (NULL);
+
 	context = g_option_context_new ("Thumbnail movies");
 	options = bacon_video_widget_get_option_group ();
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, options);
-
-	g_thread_init (NULL);
 #ifndef THUMB_DEBUG
 	g_type_init ();
 #else
-	gtk_init (&argc, &argv);
+ 	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 #endif
 
-#ifndef HAVE_GTK_ONLY
-	gnome_authentication_manager_init ();
-#endif
-
-	gnome_vfs_init ();
 	if (g_option_context_parse (context, &argc, &argv, &err) == FALSE) {
 		g_print ("couldn't parse command-line options: %s\n", err->message);
 		g_error_free (err);
 		return 1;
 	}
 
+#ifndef THUMB_DEBUG
+	if (g_fatal_warnings) {
+		GLogLevelFlags fatal_mask;
+
+		fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
+		fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
+		g_log_set_always_fatal (fatal_mask);
+	}
+#endif /* THUMB_DEBUG */
+
+	gnome_vfs_init ();
+
+#ifndef HAVE_GTK_ONLY
+	gnome_authentication_manager_init ();
+#endif
+
 	if (filenames == NULL || g_strv_length (filenames) != 2) {
 		g_print ("Expects an input and an output file\n");
 		return 1;
@@ -390,20 +482,24 @@ int main (int argc, char *argv[])
 	input = filenames[0];
 	output = filenames[1];
 
+	PROGRESS_DEBUG("Initialised libraries, about to create video widget\n");
+
 	bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new (-1, -1, BVW_USE_TYPE_CAPTURE, &err));
-	if (err != NULL)
-	{
+	if (err != NULL) {
 		g_print ("totem-video-thumbnailer couln't create the video "
 				"widget.\nReason: %s.\n", err->message);
 		g_error_free (err);
 		exit (1);
 	}
 
-	if (time_limit != FALSE)
-		g_thread_create (time_monitor, (gpointer) input, FALSE, NULL);
+	PROGRESS_DEBUG("Video widget created\n");
 
-	if (bacon_video_widget_open (bvw, input, &err) == FALSE)
-	{
+	if (time_limit != FALSE)
+		totem_resources_monitor_start (input, 0);
+
+	PROGRESS_DEBUG("About to open video file\n");
+
+	if (bacon_video_widget_open (bvw, input, &err) == FALSE) {
 		g_print ("totem-video-thumbnailer couln't open file '%s'\n"
 				"Reason: %s.\n",
 				input, err->message);
@@ -411,66 +507,39 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
+	PROGRESS_DEBUG("Opened video file: '%s'\n", input);
+	PROGRESS_DEBUG("About to play file\n");
+
 	bacon_video_widget_play (bvw, &err);
-	if (err != NULL)
-	{
+	if (err != NULL) {
 		g_print ("totem-video-thumbnailer couln't play file: '%s'\n"
 				"Reason: %s.\n", input, err->message);
 		g_error_free (err);
 		exit (1);
 	}
 
-	/* Test at multiple points in the file to see if we can get an 
-	 * interesting frame */
-	for (current = 0; current < frame_locations_length; current++)
-	{
-		length = bacon_video_widget_get_stream_length (bvw);
-		if (length > MIN_LEN_FOR_SEEK)
-		{
-			if (bacon_video_widget_seek
-					(bvw, frame_locations[current], NULL) == FALSE)
-			{
-				bacon_video_widget_play (bvw, NULL);
-			}
-		}
+	PROGRESS_DEBUG("Started playing file\n");
 
-		if (bacon_video_widget_can_get_frames (bvw, &err) == FALSE)
-		{
-			g_print ("totem-video-thumbnailer: '%s' isn't thumbnailable\n"
-					"Reason: %s\n",
-					input, err ? err->message : "programming error");
-			bacon_video_widget_close (bvw);
-			gtk_widget_destroy (GTK_WIDGET (bvw));
-			g_error_free (err);
-
-			exit (1);
-		}
-
-		/* Pull the frame, if it's interesting we bail early */
-		pixbuf = bacon_video_widget_get_current_frame (bvw);
-		if (pixbuf != NULL && is_image_interesting (pixbuf) != FALSE)
-			break;
-
-		/* If we get to the end of this loop, we'll end up using
-		 * the last image we pulled */
-		if(current + 1 < frame_locations_length) {
-			if (pixbuf != NULL)
-				g_object_unref (pixbuf);
-		}
-	}
+	/* If the user has told us to use a frame at a specific second 
+	 * into the video, just use that frame no matter how boring it
+	 * is */
+	if(second_index != -1)
+		pixbuf = capture_frame_at_time (bvw, input, output, second_index);
+	else
+		pixbuf = capture_interesting_frame (bvw, input, output);
 
 	/* Cleanup */
 	bacon_video_widget_close (bvw);
-	finished = TRUE;
+	totem_resources_monitor_stop ();
 	gtk_widget_destroy (GTK_WIDGET (bvw));
 
-	if (pixbuf == NULL)
-	{
+	if (pixbuf == NULL) {
 		g_print ("totem-video-thumbnailer couln't get a picture from "
 					"'%s'\n", input);
 		exit (1);
 	}
 
+	PROGRESS_DEBUG("Saving captured screenshot\n");
 	save_pixbuf (pixbuf, output, input, output_size, FALSE);
 	g_object_unref (pixbuf);
 
